/*
	CAView implementation
*/

#include <string.h>
#include <stdlib.h>

#include <Beep.h>
#include <Debug.h>

#include "Asteroids.h"


const float kAngleDelta = .3;


float Random( short range )
{
	return ( (float) rand() * range ) / RAND_MAX;
}


CAView::CAView( BRect frame ) :
	BView( frame, "Asteroids", B_FRAME_EVENTS | B_WILL_DRAW, 0 ),
	mTimerThread( 0 ),
	mpShip( NULL )
{
	// Make offscreen buffer
	mpOffScreen = new BView( frame, "offscreen", B_WILL_DRAW, 0 );
	mpBitmap = new BBitmap( frame, B_COLOR_8_BIT, TRUE );
	mpBitmap->AddChild( mpOffScreen );
	
	// Create background timer thread
	mTimerThread = spawn_thread( CAView::TimerFunc, "timerThread",
								B_NORMAL_PRIORITY, (void *) this );
	resume_thread( mTimerThread );
}


CAView::~CAView()
{	
	kill_thread( mTimerThread );

	// There's probably a better way to disconnect all this stuff
	if( mpOffScreen )
	{
		BWindow *pWindow = mpOffScreen->Window();
		pWindow->Lock();
		mpOffScreen->RemoveSelf();
		pWindow->Unlock();
		
		delete mpOffScreen;
	}
		
	if( mpBitmap )
	{
		delete mpBitmap;
	}
	
	if( mpShip )
		delete mpShip;
		
	CAsteroid **pAst = (CAsteroid **) mAsteroids.Items();
	for( short i=0; i<mAsteroids.CountItems(); i++ )
		delete pAst[ i ];
}


void CAView::AttachedToWindow()
{
	MakeFocus();
	SetHighColor( 102, 0, 0 );
	
	for( short i=0; i<10; i++ )
	{
		BPoint loc( Random( 640 ), Random( 480 ) );
		BPoint delta( Random( 20 ) - 10, Random( 20 ) - 10 );
		if( !delta.x )
			delta.x = 1;
		if( !delta.y )
			delta.y = 1;
		
		mAsteroids.AddItem( new CAsteroid( loc, delta, Random( 3 ) + 1 ) );
	}
	
	mpShip = new CShip( BPoint( 320, 240 ), 0 );
}


bool IsKeyDown( const key_info& keystate, char key )
{
	return ( keystate.key_states[ key >> 3 ] & ( 1 << ( 7 - ( key % 8 ) ) ) );
}


bool CAView::Tick()
{
	if( !mAsteroids.CountItems() )
	{
		static short waitClock = -1;
		
		if( waitClock == 0 )
		{ 
			for( short i=0; i<10; i++ )
			{
				BPoint loc( Random( 640 ), Random( 480 ) );
				BPoint delta( Random( 20 ) - 10, Random( 20 ) - 10 );
				if( !delta.x )
					delta.x = 1;
				if( !delta.y )
					delta.y = 1;
				
				mAsteroids.AddItem( new CAsteroid( loc, delta, Random( 3 ) + 1 ) );
			}
			
			waitClock = -1;
		}
		else if( waitClock == -1 )
			waitClock = 35;
		else
			waitClock--;
	}
	
	// Handle asteroids
	BRect bounds = Bounds();
	CAsteroid **pAst = (CAsteroid **) mAsteroids.Items();
	for( short i=0; i<mAsteroids.CountItems(); i++ )
		pAst[ i ]->Move( bounds );
		
	for( short i=0; i<mAsteroids.CountItems() - 1; i++ )
		for( short j=i + 1; j<mAsteroids.CountItems(); j++ )
			pAst[ i ]->Collide( pAst[ j ] );
		
	// Handle player
	if( mpShip )
	{
		key_info keys;
		BPoint delta( 0, 0 );
		
		GetKeys( &keys, FALSE );
		
		if( IsKeyDown( keys, 97 ) )		// left
			mpShip->Turn( -kAngleDelta );
		if( IsKeyDown( keys, 99 ) )		// right
			mpShip->Turn( kAngleDelta );
		if( IsKeyDown( keys, 87 ) )		// up
			mpShip->Thrust();
		if( IsKeyDown( keys, 94 ) )		// space
			mpShip->Shoot();
			
		mpShip->Move( bounds );
		mpShip->Collide( mAsteroids );
	}
			
	return TRUE;
}


void CAView::Render()
{
	// put objects on the offscreen
	mpOffScreen->Window()->Lock();
	memset( mpBitmap->Bits(), 255, mpBitmap->BitsLength() );
	
	// Handle asteroids
	BRect bounds = Bounds();
	CAsteroid **pAst = (CAsteroid **) mAsteroids.Items();
	for( short i=0; i<mAsteroids.CountItems(); i++, pAst++ )
		(*pAst)->Draw( mpOffScreen );

	// Handle player
	if( mpShip )
		mpShip->Draw( mpOffScreen );		

	mpOffScreen->Sync();
	mpOffScreen->Window()->Unlock();
	
	// Put on screen
	Draw( Bounds() );
}


void CAView::Draw( BRect updateRect )
{
	DrawBitmap( mpBitmap );
}


const kSleep = 50000;

long CAView::TimerFunc( void *pPtr )
{
	CAView *pView = (CAView *) pPtr;
	
	double lastTime = system_time();
	
	while( 1 )
	{
		double thisTime = system_time();
		
		double sleep = kSleep - ( thisTime - lastTime );
		lastTime = thisTime;
		
		// If we sleep first, the window and stuff has time to initialize
		if( sleep > 0 )
			snooze( sleep );
		
		pView->Window()->Lock();
		pView->Tick();
		pView->Render();
		pView->Window()->Unlock();
	}
	
	return 0;
}


