/*
	Asteroid.cpp
	
	Implementation of a CAsteroid
*/


#include <stdio.h>
#include "Asteroids.h"

short gRadii[ 5 ] = {10, 20, 40, 60, 80 };


CAsteroid::CAsteroid( BPoint loc, BPoint delta, short shape ) :
	mLoc( loc ),
	mDelta( delta ),
	mShape( shape )
{
}


void CAsteroid::Move( const BRect& bounds )
{
	float r = gRadii[ mShape ];
	
	mLoc += mDelta;
	
	if( mLoc.x < -r )
		mLoc.x = bounds.right + r;
	else if( mLoc.x > bounds.right + r )
		mLoc.x = -r;
		
	if( mLoc.y < -r )
		mLoc.y = bounds.bottom + r;
	else if( mLoc.y > bounds.bottom + r )
		mLoc.y = -r;
}


void CAsteroid::Collide( CAsteroid *pAst )
{
	float r = gRadii[ mShape ];
	BRect b1( mLoc.x - r, mLoc.y - r, mLoc.x + r, mLoc.y + r );
	r = gRadii[ pAst->mShape ];
	BRect b2( pAst->mLoc.x - r, pAst->mLoc.y - r, pAst->mLoc.x + r, pAst->mLoc.y + r ); 	
	
	if( b1.Intersects( b2 ) )
	{
		BPoint r = mLoc - pAst->mLoc;
		float dist1 = abs( r.x ) + abs( r.y );
		r = ( mLoc + mDelta ) - ( pAst->mLoc + pAst->mDelta );
		float dist2 = abs( r.x ) + abs( r.y );
		
		if( dist2 < dist1 )
		{
			BPoint temp;
			
			temp = mDelta;
			mDelta = pAst->mDelta;
			pAst->mDelta = temp;
		}
	}
}


void CAsteroid::Draw( BView *pView )
{
	pView->FillEllipse( mLoc, gRadii[ mShape ], gRadii[ mShape ], &B_MIXED_COLORS );
}




