/*
	Classes for Gui terminal program
*/


#include <Application.h>
#include <Window.h>
#include <View.h>
#include <Bitmap.h>


class CAApp : BApplication
{
	public:
		CAApp();
		virtual ~CAApp();
};
	

class CAWindow : public BWindow
{
	public:
		CAWindow();
		virtual ~CAWindow();

		virtual	bool QuitRequested();
};


class CAsteroid
{
	public:
		CAsteroid( BPoint loc, BPoint delta, short shape );
		
		void Move( const BRect& bounds );
		void Collide( CAsteroid *pAst );
		void Draw( BView *pView );
		
	public:
		BPoint mLoc;
		BPoint mDelta;
		short mShape;
};


class CShot
{
	public:
		CShot();
		CShot( BPoint loc, BPoint vector, short timer );
		
		bool Move( const BRect& bounds );
		void Draw( BView *pView );
		
	public:
		bool mActive;
		BPoint mLoc;
		BPoint mDelta;
		
		short mTimer;
};


const short kMaxShots = 32;
		
class CShip
{
	public:
		CShip( BPoint loc, float angle );
		
		void Move( const BRect& bounds );
		void Collide( BList& asteroids );
		void Turn( float angle );
		void Thrust();
		void Shoot();
		void Draw( BView *pView );
		
	public:
		BPoint mLoc;
		BPoint mDelta;
		BPoint mVector;
		float mAngle;
		
		CShot mShots[ kMaxShots ];
};
		

class CAView : public BView
{
	public:
		CAView( BRect frame );
		virtual ~CAView();
				
		virtual	void AttachedToWindow();
		
		virtual bool Tick();
		virtual void Render();
		virtual	void Draw( BRect updateRect );
		
	private:
		BBitmap *mpBitmap;
		BView *mpOffScreen;	
		thread_id mTimerThread;

		BList mAsteroids;
		CShip *mpShip;
		
	private:
		static long TimerFunc( void *pPtr );
};
	

inline
float abs( float val )
{
	return val < 0 ? -val : val;
}

