/*
	ship.cpp
*/

#include <stdio.h>

#include "Asteroids.h"


const float kMaxVel = 40;
const float kThrust = 3;
const float kRadius = 15;

extern short gRadii[];


CShot::CShot() :
	mLoc( 0, 0 ),
	mDelta( 0, 0 ),
	mTimer( 0 ),
	mActive( FALSE )
{
}


CShot::CShot( BPoint loc, BPoint vector, short timer ) :
	mLoc( loc ),
	mDelta( vector ),
	mTimer( timer ),
	mActive( FALSE )
{
}


bool CShot::Move( const BRect& bounds )
{
	mLoc += mDelta;
	if( mLoc.x < 0 )
		mLoc.x = bounds.right;
	else if( mLoc.x > bounds.right )
		mLoc.x = 0;
		
	if( mLoc.y < 0 )
		mLoc.y = bounds.bottom;
	else if( mLoc.y > bounds.bottom )
		mLoc.y = 0;
		
	return --mTimer;
}


void CShot::Draw( BView *pView )
{
	pView->StrokeLine( mLoc, mLoc );
}


CShip::CShip( BPoint loc, float angle ) :
	mLoc( loc ),
	mDelta( 0, 0 ),
	mAngle( angle )
{
}


void CShip::Move( const BRect& bounds )
{
	mLoc += mDelta;
	mDelta.x *= .99;
	mDelta.y *= .99;
	
	if( abs( mDelta.x ) < .5 )
		mDelta.x = 0;
	if( abs( mDelta.y ) < .5 )
		mDelta.y = 0;

	float r = 20;
	if( mLoc.x < -r )
		mLoc.x = bounds.right + r;
	else if( mLoc.x > bounds.right + r )
		mLoc.x = -r;
		
	if( mLoc.y < -r )
		mLoc.y = bounds.bottom + r;
	else if( mLoc.y > bounds.bottom + r )
		mLoc.y = -r;
		
	// Process shots
	for( short i=0; i<kMaxShots; i++ )
	{
		{
			if( !mShots[ i ].Move( bounds ) )
				mShots[ i ].mActive = FALSE;
		}
	}
}

float dist2( BPoint& p1, BPoint& p2 )
{
	BPoint temp = p2 - p1;
	
	return temp.x * temp.x + temp.y * temp.y;
}


void CShip::Collide( BList& asteroids )
{
	// Process shots
	for( short i=0; i<kMaxShots; i++ )
	{
		if( mShots[ i ].mActive )
		{	
			short aCount = asteroids.CountItems();
			for( short j=0; j<aCount; j++ )
			{
				CAsteroid *pAst = (CAsteroid *) asteroids.ItemAt( j );
				float r = gRadii[ pAst->mShape ];
				if( dist2( mShots[ i ].mLoc, pAst->mLoc ) <= r * r )
				{
					// We have a collision...make new rocks
					if( !pAst->mShape )
					{
						delete pAst;
						asteroids.RemoveItem( pAst );
						break;
					}
					else
					{
						pAst->mShape--;
						mShots[ i ].mActive = FALSE;
						
						BPoint loc = pAst->mLoc;
						BPoint delta = pAst->mDelta;
						short shape = pAst->mShape;
						
						CAsteroid *pRock = new CAsteroid( loc, delta, shape );
						pRock->mDelta.x *= -1;
						pRock->mDelta.y *= -1;
						asteroids.AddItem( pRock );
					}
					
				}
			}
		}
	
	}
	
}


void CShip::Turn( float angle )
{
	mAngle += angle;
	
	mVector.x = cos( mAngle ) * kThrust;
	mVector.y = sin( mAngle ) * kThrust;
}


void CShip::Thrust()
{
	if( mDelta.x < kMaxVel && mDelta.y < kMaxVel )
		mDelta += mVector;
}


void CShip::Shoot()
{
	short i;
	for( i=0; i<kMaxShots; i++ )
		if( !mShots[ i ].mActive )
			break;
			
	if( i == kMaxShots )
		return;
		
	CShot *pShot = &mShots[ i ];
	BPoint b( cos( mAngle ) * kRadius + 2, sin( mAngle ) * kRadius + 2 );
	b += mLoc;
	pShot->mLoc = b;
	BPoint d = mVector;
	d.x *= 2;
	d.y *= 2;
	
	// Adding in mDelta just makes the shots go out infront of the ship
	pShot->mDelta = d + mDelta;
	pShot->mTimer = kMaxShots + 1;
	pShot->mActive = TRUE;
}


void CShip::Draw( BView *pView )
{
	pView->StrokeEllipse( mLoc, kRadius, kRadius );
	BPoint b( cos( mAngle ) * kRadius, sin( mAngle ) * kRadius );
	b += mLoc;
	
	pView->StrokeEllipse( b, kRadius / 5, kRadius / 5 );

	// Process shots
	for( short i=0; i<kMaxShots; i++ )
	{
		if( mShots[ i ].mActive )	
			mShots[ i ].Draw( pView );
	}
}



